@@ -1,16 +1,22 @@
 package Net::DNS::Header;
 #
-# $Id: Header.pm,v 2.100 2003/12/13 01:37:04 ctriv Exp $
+# $Id: Header.pm 388 2005-06-22 10:06:05Z olaf $
 #
 
 use strict;
+
+BEGIN { 
+    eval { require bytes; }
+} 
+
+
 use vars qw($VERSION $AUTOLOAD);
 
 use Net::DNS;
 
 use constant MAX_ID => 65535;
 
-$VERSION = (qw$Revision: 2.100 $)[1];
+$VERSION = (qw$LastChangedRevision: 388 $)[1];
 
 =head1 NAME
 
@@ -58,7 +64,7 @@ sub new {
 	if (@_) {
 		my $data = shift;
 
-		if (length($$data) < &Net::DNS::HFIXEDSZ) {
+		if (length($$data) < Net::DNS::HFIXEDSZ()) {
 			return undef;
 		}
 
@@ -353,7 +359,7 @@ sub data {
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
-Portions Copyright (c) 2002-2003 Chris Reinhardt.
+Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
